@echo off

::
:: This script installs uberAgent silently
::
:: It is a method of installing uberAgent through Splunk's Deployment server
:: Alternatively you can simply install the MSI through other means
:: 
:: Requirements:
::
:: - The variable %servers% below MUST HAVE THE CORRECT VALUE FOR YOUR ENVIRONMENT!
::
:: - By default data is sent to Splunk via TCP
:: - In order to send data via HTTP(S) to Splunk's HTTP Event Collector the following variables need to be defined:
::    - %resttoken%
::    - %protocol%
::
:: - The following files must be present in the same directory as this batch file
::    - uberAgent-32.msi
::    - uberAgent-64.msi
::
:: Return values:
::
::    1  ->    Required file is missing
::    x  ->    Return value from msiexec
::

:::::::::::::::::::::::::::::::::::::::::
::
:: Variables - DO edit
::
:::::::::::::::::::::::::::::::::::::::::

set servers=localhost:19500

::
:: The following variables are only required if you want uberAgent to send data to Splunk's HTTP Event Collector
:: IMPORTANT: encrypted tokens start with a percent sign which must be escaped by prepending another percent sign.
::
:: set protocol=HTTP
:: set resttoken=YOUR_REST_TOKEN_HERE

::
:: Set the following variable to 1, if you want to register uberAgent's Internet Explorer add-on automatically
::
set RegisterIEAddOn=0

:::::::::::::::::::::::::::::::::::::::::
::
:: Start of script
::
:::::::::::::::::::::::::::::::::::::::::

:: Check prerequisites
if not exist "%~dp0uberAgent-32.msi" (
   echo ERROR 32-bit MSI not found >&2
	exit /b 1
)
if not exist "%~dp0uberAgent-64.msi" (
   echo ERROR 64-bit MSI not found >&2
	exit /b 1
)

:: Build the MSI parameter string
set msiParamString=SERVERS="%servers%"
if defined protocol (
   set msiParamString=%msiParamString% RECEIVER_PROTOCOL=%protocol%
)
if defined resttoken (
   set msiParamString=%msiParamString% REST_TOKEN=%resttoken%
)

::
:: Select 32- or 64-bit version according to the bitness of the OS
::
if defined programfiles(x86) (
   set installer=%~dp0uberAgent-64.msi
) else (
   set installer=%~dp0uberAgent-32.msi
)

::
:: Copy license file and custom configuration files
::
if not exist "%ProgramData%\vast limits" (
   mkdir "%ProgramData%\vast limits"
   icacls "%ProgramData%\vast limits" /inheritance:r /grant:r "*S-1-5-32-544:(OI)(CI)F" /grant:r "*S-1-5-18:(OI)(CI)F"
)   

if exist "%~dp0uberAgent*.lic" (
   xcopy "%~dp0uberAgent*.lic" "%ProgramData%\vast limits\uberAgent\License cache\" /y /s /v /e
)

if exist "%~dp0uberAgent*.conf" (
   xcopy "%~dp0uberAgent*.conf" "%ProgramData%\vast limits\uberAgent\Configuration\" /y /s /v /e
)

::
:: Run the installer
::
echo INFO Running the installer... >&2

start /wait msiexec /i "%installer%" /quiet /l*v "%temp%\uberAgentMsi.log" %msiParamString% 1>nul

::
:: Register IE browser helper objects (BHO)
::
if "%RegisterIEAddOn%" == "1" (
   if exist "%ProgramFiles%\vast limits\uberAgent\uAIEExtension32.dll" (
      regsvr32 /s "%ProgramFiles%\vast limits\uberAgent\uAIEExtension32.dll"
   )
   if exist "%ProgramFiles%\vast limits\uberAgent\uAIEExtension64.dll" (
      regsvr32 /s "%ProgramFiles%\vast limits\uberAgent\uAIEExtension64.dll"
   )
)

echo INFO ...done! >&2

exit /b %errorlevel%