//
// Token change event handler
//
// RequireJS dependency handling
require (["jquery", "splunkjs/mvc", "splunkjs/mvc/simplexml/ready!"], function ($, mvc)
{
   // Get the default token model
   var defaultTokenModel = mvc.Components.get ("default");
   if (!defaultTokenModel)
   {
      console.log ("Error: default token model unavailable!");
      return;
   }
   
   // Bind event listener for future changes
   defaultTokenModel.on ("change:RootProcGUID", function (model, value, options)
   {
      // Update the $RootProcGUIDString$ token value
      defaultTokenModel.set("RootProcHost", "");
      defaultTokenModel.set("RootProcAppString", "");
      defaultTokenModel.set("RootProcCmdline", "");
      defaultTokenModel.set("RootProcUser", "");
      defaultTokenModel.set("RootProcParentString", "");
      defaultTokenModel.set("RootProcParentGUID", "");
      defaultTokenModel.set("RootProcIsElevated", "");
   });
});

//
// Table cell renderer
//
require(['jquery', 'underscore', 'splunkjs/mvc', 'splunkjs/mvc/tableview', 'splunkjs/mvc/simplexml/ready!'], function($, _, mvc, TableView)
{
   var TimebarCellRenderer = TableView.BaseCellRenderer.extend(
   {
      // Called to check whether to invoke our custom renderer
      canRender: function(cell)
      {
         return (cell.field === 'Lifetime' || cell.field === 'Command line');
      },
      
      // The render function is only called when canRender returns true
      render: function($td, cell)
      {
         if (cell.value === null)
         {
            return;
         }

         if (cell.field === 'Lifetime')
         {
            var elements = cell.value.split (";");
            if (elements.length != 4)
            {
               return;
            }
            var startRelativeTime = elements[0];
            var lifetime = elements[1];
            var endRelativeTime = parseInt(startRelativeTime, 10) + parseInt(lifetime, 10);
            var totalDuration = elements[2];
            var processType = elements[3];
            
            var color = "#1e93c6";
            if (processType === "User profile")
            {
               color = "#99712b";
            }
            else if (processType === "Session teardown")
            {
               color = "#d6563c";
            }
            else if (processType === "GP logoff script")
            {
               color = "#6a5c9e";
            }
            
            $td.addClass('timebar-cell').html(_.template('<div title="<%- tooltip %>" class="timebar-wrapper"><div class="timebar" style="margin-left:<%- leftPercent %>%; width:<%- widthPercent %>%; background-color:<%- color %>;"></div></div>',
            {
               leftPercent:   startRelativeTime/totalDuration*100,
               widthPercent:  lifetime/totalDuration*100,
               tooltip:       "Relative start time (ms): " + startRelativeTime + "\n" + "Lifetime (ms): " + lifetime + "\n" + "Relative end time (ms): " + endRelativeTime,
               color:         color
            }));
         }
         else if (cell.field === 'Command line')
         {
            var message = cell.value;
            var tooltip = cell.value;
            
            message = message.replace ("\\SystemRoot\\System32\\", "...\\");
            message = message.replace ("%SystemRoot%\\system32\\", "...\\");
            message = message.replace ("\\??\\C:\\Windows\\system32\\", "...\\");
            message = message.replace ("C:\\Windows\\system32\\", "...\\");
            message = message.replace ("C:\\Program Files (x86)\\", "...\\");
            message = message.replace ("C:\\Program Files\\", "...\\");
            message = message.replace ("C:\\Windows\\SysWOW64\\", "...\\");
            message = message.replace ("C:\\Windows\\", "...\\");
            if (message.length > 50)
            {
               message = message.substring(0, 49) + "...";
            }
               
            $td.html(_.template('<div title="<%- tooltip %>"> <%- text %></div>',
            {
               text:       message,
               tooltip:    tooltip
            }));
         }
      }
   });

   mvc.Components.get('Table_Panel41').getVisualization(function(tableView)
   {
      // Register custom cell renderer
      tableView.table.addCellRenderer(new TimebarCellRenderer());
      
      // Force the table to re-render
      tableView.table.render();
   });
});